#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
ISLI demographics initialization
"""

from schooltool.app.app import InitBase
from schooltool.basicperson.demographics import BoolFieldDescription
from schooltool.basicperson.demographics import DescriptionFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.demographics import IntFieldDescription
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.interfaces import IDemographicsFields

from schooltool.isli import IsliMessage as _


def setUpDemographics(app):
    dfs = IDemographicsFields(app)

    # student fields
    if 'class_roll_number' not in dfs:
        dfs['class_roll_number'] = IntFieldDescription(
            'class_roll_number', _('Class roll number'),
            limit_keys=['students'])

    if 'language' not in dfs:
        dfs['language'] = EnumFieldDescription(
            'language', _('Language (Mother tongue)'),
            limit_keys=['students'])
        dfs['language'].items = [
            _('Assamese (Asamiya)'),
            _('Bengali'),
            _('Bodo'),
            _('Dogri'),
            _('Gujarati'),
            _('Hindi'),
            _('Kannada'),
            _('Kashmiri'),
            _('Konkani'),
            _('Maithili'),
            _('Malayalam'),
            _('Manipuri'),
            _('Marathi'),
            _('Nepali'),
            _('Oriya'),
            _('Punjabi'),
            _('Sanskrit'),
            _('Santali'),
            _('Sindhi'),
            _('Tamil'),
            _('Telugu'),
            _('Urdu'),
            _('Other'),
        ]

    if 'religion' not in dfs:
        dfs['religion'] = EnumFieldDescription(
            'religion', _('Religion'),
            limit_keys=['students'])
        dfs['religion'].items = [
            _('Hindu'),
            _('Muslim'),
            _('Christian'),
            _('Jain'),
            _('Sikh'),
            _('Buddhist'),
            _('Other'),
        ]

    if 'caste' not in dfs:
        dfs['caste'] = EnumFieldDescription(
            'caste', _('Caste'),
            limit_keys=['students'])
        dfs['caste'].items = [
            _('General'),
            _('ST'),
            _('SC'),
            _('OBC'),
        ]

    if 'economic_weakness' not in dfs:
        dfs['economic_weakness'] = EnumFieldDescription(
            'economic_weakness', _('EWS / Freeship'),
            limit_keys=['students'])
        dfs['economic_weakness'].items = [
            _('ST'),
            _('SC'),
            _('OBC'),
            _('CWSN'),
            _('Orphan'),
            _('Transgender'),
            _('EWS'),
        ]

    if 'blood_group' not in dfs:
        dfs['blood_group'] = TextFieldDescription(
            'blood_group', _('Blood group'),
            limit_keys=['students'])

    if 'disability' not in dfs:
        dfs['disability'] = EnumFieldDescription(
            'disability', _('Disability?'),
            limit_keys=['students'])
        dfs['disability'].items = [_('No'),
                                   _('Yes - sight'),
                                   _('Yes - hearing'),
                                   _('Yes - mobility'),
                                   _('Yes - other')]

    if 'dietary_requirements' not in dfs:
        dfs['dietary_requirements'] = TextFieldDescription(
            'dietary_requirements',
            _('Dietary requirements / medical conditions'),
            limit_keys=['students'])

    if 'distance' not in dfs:
        dfs['distance'] = IntFieldDescription(
            'distance', _('Distance travelled to school (minutes)'),
            limit_keys=['students'])

    if 'kindergarten_exam' not in dfs:
        dfs['kindergarten_exam'] = IntFieldDescription(
            'kindergarten_exam', _('Kindergarten Leaver Exam'),
            limit_keys=['students'])

    if 'end_of_primary_exam' not in dfs:
        dfs['end_of_primary_exam'] = IntFieldDescription(
            'end_of_primary_exam', _('End of primary Exam'),
            limit_keys=['students'])

    if 'mother_name' not in dfs:
        dfs['mother_name'] = TextFieldDescription(
            'mother_name', _('Mother name'),
            limit_keys=['students'])

    if 'father_name' not in dfs:
        dfs['father_name'] = TextFieldDescription(
            'father_name', _('Father name'),
            limit_keys=['students'])

    # teacher fields
    if 'shift_type' not in dfs:
        dfs['shift_type'] = EnumFieldDescription(
            'shift_type', _('Shift type'),
            limit_keys=['teachers'])
        dfs['shift_type'].items = [_('Full time'),
                                   _('Part time')]

    if 'joining_year' not in dfs:
        dfs['joining_year'] = IntFieldDescription(
            'joining_year', _('Year of joining the school'),
            limit_keys=['teachers'])

    if 'subject_1' not in dfs:
        dfs['subject_1'] = TextFieldDescription(
            'subject_1', _('Subject taught (1)'),
            limit_keys=['teachers'])

    if 'subject_2' not in dfs:
        dfs['subject_2'] = TextFieldDescription(
            'subject_2', _('Subject taught (2)'),
            limit_keys=['teachers'])

    if 'subject_3' not in dfs:
        dfs['subject_3'] = TextFieldDescription(
            'subject_3', _('Subject taught (3)'),
            limit_keys=['teachers'])

    if 'classes' not in dfs:
        dfs['classes'] = TextFieldDescription(
            'classes', _('Classes taught'),
            limit_keys=['teachers'])

    if 'qualifications' not in dfs:
        dfs['qualifications'] = EnumFieldDescription(
            'qualifications', _('Qualifications'),
            limit_keys=['teachers'])
        dfs['qualifications'].items = [_('Graduate'),
                                       _('Diploma'),
                                       _('S6 graduate'),
                                       _('Unqualified')]

    if 'other_responsibilities' not in dfs:
        dfs['other_responsibilities'] = EnumFieldDescription(
            'other_responsibilities', _('Other Responsibilities'),
            limit_keys=['teachers'])
        dfs['other_responsibilities'].items = [
            _('Principal'),
            _('Head of teaching and learning'),
            _('Head of a department'),
            _('SchoolTool core team'),
            _('Class teacher'),
        ]

    if 'cpd_attended' not in dfs:
        dfs['cpd_attended'] = DescriptionFieldDescription(
            'cpd_attended', _('CPD attended'),
            limit_keys=['teachers'])

    if 'email' not in dfs:
        dfs['email'] = TextFieldDescription(
            'email', _('Email'),
            limit_keys=['teachers'])

    if 'phone' not in dfs:
        dfs['phone'] = TextFieldDescription(
            'phone', _('Phone'),
            limit_keys=['students', 'teachers'])

    if 'address_street' not in dfs:
        dfs['address_street'] = TextFieldDescription(
            'address_street', _('Address: Street'),
            limit_keys=['students', 'teachers'])

    if 'address_city' not in dfs:
        dfs['address_city'] = TextFieldDescription(
            'address_city', _('Address: City'),
            limit_keys=['students', 'teachers'])

    if 'address_state' not in dfs:
        dfs['address_state'] = TextFieldDescription(
            'address_state', _('Address: State'),
            limit_keys=['students', 'teachers'])

    if 'address_pincode' not in dfs:
        dfs['address_pincode'] = TextFieldDescription(
            'address_pincode', _('Address: Pincode'),
            limit_keys=['students', 'teachers'])


class IsliDemographicsInit(InitBase):

    after = ('schooltool.basicperson.demographics',)

    def __call__(self):
        self.deleteDefaultFields()
        setUpDemographics(self.app)

    def deleteDefaultFields(self):
        dfs = IDemographicsFields(self.app)
        names = ['ID', 'ethnicity', 'language', 'placeofbirth', 'citizenship']
        for name in names:
            if name in dfs:
                del dfs[name]
