#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2015 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 3.

Reverts caste demographics field update and updates economic weekness field
"""

from zope.app.generations.utility import getRootFolder

from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription

from schooltool.isli import IsliMessage as _


def revert(field):
    field.title = _('Caste')
    field.items = [
        _('General'),
        _('ST'),
        _('SC'),
        _('OBC'),
    ]


def evolve(context):
    app = getRootFolder(context)
    dfs = IDemographicsFields(app)
    field = dfs.get('caste')
    if field is not None:
        if field.title == _('EWS / Freeship') or _('CWSN') in field.items:
            revert(field)
            field._p_changed = True
    name = 'economic_weakness'
    if name in dfs and isinstance(dfs[name], TextFieldDescription):
        del dfs[name]
        original_order = dfs._order[:]
        dfs[name] = EnumFieldDescription(name, _('EWS / Freeship'),
                                         limit_keys=['students'])
        dfs[name].items = [
            _('ST'),
            _('SC'),
            _('OBC'),
            _('CWSN'),
            _('Orphan'),
            _('Transgender'),
            _('EWS'),
        ]
        dfs[name]._p_changed = True
        if 'caste' in dfs:
            caste_index = original_order.index('caste')
            ews_index = caste_index + 1
            new_order = tuple(original_order[:ews_index] + [name] +
                              original_order[ews_index:])
            dfs.updateOrder(new_order)
