#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2016 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 7.

Synchronizes stream-sections for students in the current year
"""

from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite

from schooltool.group.interfaces import IGroupContainer
from schooltool.relationship.temporal import ACTIVE
from schooltool.relationship.temporal import ACTIVE_CODE
from schooltool.relationship.temporal import INACTIVE
from schooltool.relationship.temporal import INACTIVE_CODE
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.stream.stream import StreamMembers
from schooltool.term.interfaces import ITerm


def sync(student, schoolyear):
    relationships = StreamMembers.bind(member=student).all().relationships
    for link_info in relationships:
        stream = link_info.target
        if ISchoolYear(stream) == schoolyear:
            state_accessor = link_info.state
            state_date, meaning, code = state_accessor.all()[-1]
            if ACTIVE_CODE in meaning:
                for section in stream.sections:
                    term_start = ITerm(section).first
                    if term_start > state_date:
                        enrollment_date = term_start
                    else:
                        enrollment_date = state_date
                    if student not in section.members.on(enrollment_date):
                        section.members.on(enrollment_date).relate(
                            student, ACTIVE, ACTIVE_CODE)
            elif INACTIVE_CODE in meaning:
                for section in stream.sections:
                    term_start = ITerm(section).first
                    if term_start > state_date:
                        enrollment_date = term_start
                    else:
                        enrollment_date = state_date
                    if student in section.members.on(enrollment_date):
                        section.members.on(enrollment_date).relate(
                            student, INACTIVE, INACTIVE_CODE)


def evolve(context):
    app = getRootFolder(context)
    oldSite = getSite()
    setSite(app)
    schoolyears = ISchoolYearContainer(app)
    schoolyear = schoolyears.getActiveSchoolYear()
    if schoolyear is None:
        return
    students = IGroupContainer(schoolyear)['students']
    for student in students.members:
        sync(student, schoolyear)
    setSite(oldSite)
